export type Node = import('./base-node.js').Node;
export type URLArtifact = Omit<LH.Artifacts['URL'], 'finalDisplayedUrl'>;
export type NetworkNodeOutput = {
    nodes: Array<NetworkNode>;
    idToNodeMap: Map<string, NetworkNode>;
    urlToNodeMap: Map<string, Array<NetworkNode>>;
    frameIdToNodeMap: Map<string, NetworkNode | null>;
};
export class PageDependencyGraph {
    /**
     * @param {Lantern.NetworkRequest} request
     * @return {Array<string>}
     */
    static getNetworkInitiators(request: Lantern.NetworkRequest): Array<string>;
    /**
     * @param {Array<Lantern.NetworkRequest>} networkRequests
     * @return {NetworkNodeOutput}
     */
    static getNetworkNodeOutput(networkRequests: Array<Lantern.NetworkRequest>): NetworkNodeOutput;
    /**
     * @param {LH.TraceEvent[]} mainThreadEvents
     * @return {Array<CPUNode>}
     */
    static getCPUNodes(mainThreadEvents: LH.TraceEvent[]): Array<CPUNode>;
    /**
     * @param {NetworkNode} rootNode
     * @param {NetworkNodeOutput} networkNodeOutput
     */
    static linkNetworkNodes(rootNode: NetworkNode, networkNodeOutput: NetworkNodeOutput): void;
    /**
     * @param {Node} rootNode
     * @param {NetworkNodeOutput} networkNodeOutput
     * @param {Array<CPUNode>} cpuNodes
     */
    static linkCPUNodes(rootNode: Node, networkNodeOutput: NetworkNodeOutput, cpuNodes: Array<CPUNode>): void;
    /**
     * Removes the given node from the graph, but retains all paths between its dependencies and
     * dependents.
     * @param {Node} node
     */
    static _pruneNode(node: Node): void;
    /**
     * TODO(15841): remove when CDT backend is gone. until then, this is a useful debugging tool
     * to find delta between using CDP or the trace to create the network requests.
     *
     * When a test fails using the trace backend, I enabled this debug method and copied the network
     * requests when CDP was used, then when trace is used, and diff'd them. This method helped
     * remove non-logical differences from the comparison (order of properties, slight rounding
     * discrepancies, removing object cycles, etc).
     *
     * When using for a unit test, make sure to do `.only` so you are getting what you expect.
     * @param {Lantern.NetworkRequest[]} lanternRequests
     * @return {never}
     */
    static _debugNormalizeRequests(lanternRequests: Lantern.NetworkRequest[]): never;
    /**
     * @param {LH.TraceEvent[]} mainThreadEvents
     * @param {Lantern.NetworkRequest[]} networkRequests
     * @param {URLArtifact} URL
     * @return {Node}
     */
    static createGraph(mainThreadEvents: LH.TraceEvent[], networkRequests: Lantern.NetworkRequest[], URL: URLArtifact): Node;
    /**
     * @param {Lantern.NetworkRequest} request The request to find the initiator of
     * @param {Map<string, Lantern.NetworkRequest[]>} requestsByURL
     * @return {Lantern.NetworkRequest|null}
     */
    static chooseInitiatorRequest(request: Lantern.NetworkRequest, requestsByURL: Map<string, Lantern.NetworkRequest[]>): Lantern.NetworkRequest | null;
    /**
     * Returns a map of `pid` -> `tid[]`.
     * @param {LH.Trace} trace
     * @return {Map<number, number[]>}
     */
    static _findWorkerThreads(trace: LH.Trace): Map<number, number[]>;
    /**
     * @param {URL|string} url
     */
    static _createParsedUrl(url: URL | string): {
        scheme: string;
        host: string;
        securityOrigin: string;
    };
    /**
     * @param {LH.Artifacts.TraceEngineResult} traceEngineResult
     * @param {Map<number, number[]>} workerThreads
     * @param {import('@paulirish/trace_engine/models/trace/types/TraceEvents.js').SyntheticNetworkRequest} request
     * @return {Lantern.NetworkRequest=}
     */
    static _createLanternRequest(traceEngineResult: LH.Artifacts.TraceEngineResult, workerThreads: Map<number, number[]>, request: import('@paulirish/trace_engine/models/trace/types/TraceEvents.js').SyntheticNetworkRequest): Lantern.NetworkRequest | undefined;
    /**
     *
     * @param {Lantern.NetworkRequest[]} lanternRequests
     */
    static _linkInitiators(lanternRequests: Lantern.NetworkRequest[]): void;
    /**
     * @param {LH.TraceEvent[]} mainThreadEvents
     * @param {LH.Trace} trace
     * @param {LH.Artifacts.TraceEngineResult} traceEngineResult
     * @param {LH.Artifacts.URL} URL
     */
    static createGraphFromTrace(mainThreadEvents: LH.TraceEvent[], trace: LH.Trace, traceEngineResult: LH.Artifacts.TraceEngineResult, URL: LH.Artifacts.URL): Promise<{
        graph: Node;
        records: Lantern.NetworkRequest<any>[];
    }>;
    /**
     *
     * @param {Node} rootNode
     */
    static printGraph(rootNode: Node, widthInCharacters?: number): void;
}
import { NetworkNode } from './network-node.js';
import * as Lantern from './types/lantern.js';
import { CPUNode } from './cpu-node.js';
//# sourceMappingURL=page-dependency-graph.d.ts.map